class TestExecution(unittest.TestCase):
    def setUp(self):
        self.ml_instance = DerivaML(
            hostname, test_catalog.catalog_id, SNAME_DOMAIN, None, None, "1"
        )
        self.domain_schema = self.ml_instance.model.schemas[SNAME_DOMAIN]
        self.model = self.ml_instance.model
        self.files = os.path.dirname(__file__) + "/files"

    def test_upload_configuration(self):
        populate_test_catalog(self.ml_instance, SNAME_DOMAIN)
        config_file = self.files + "/test-workflow-1.json"
        config = ExecutionConfiguration.load_configuration(config_file)
        rid = self.ml_instance.upload_execution_configuration(config)
        self.assertEqual(rid, self.ml_instance.retrieve_rid(rid)["RID"])

    def test_execution_1(self):
        populate_test_catalog(self.ml_instance, SNAME_DOMAIN)
        exec_config = ExecutionConfiguration.load_configuration(
            self.files + "/test-workflow-1.json"
        )
        configuration_record = self.ml_instance.initialize_execution(
            configuration=exec_config
        )
        with self.ml_instance.execution(configuration=configuration_record) as exec:
            output_dir = self.ml_instance.execution_assets_path / "testoutput"
            output_dir.mkdir(parents=True, exist_ok=True)
            with open(output_dir / "test.txt", "w+") as f:
                f.write("Hello there\n")
        upload_status = self.ml_instance.upload_execution(
            configuration=configuration_record
        )
        e = (
            list(
                self.ml_instance.catalog.getPathBuilder()
                .deriva_ml.Execution.entities()
                .fetch()
            )
        )[0]
        self.assertEqual(e["Status"], "Completed")



import unittest
from os import write

from deriva_ml import DerivaML
from deriva.core import DerivaServer, get_credential
import os
from pathlib import Path
from tempfile import TemporaryDirectory
from random import random
from deriva_ml.demo_catalog import (
    create_ml_schema,
    create_domain_schema,
    reset_demo_catalog,
)
import logging

hostname = os.getenv("DERIVA_PY_TEST_HOSTNAME")
SNAME = os.getenv("DERIVA_PY_TEST_SNAME")
SNAME_DOMAIN = "deriva-test"

logger = logging.getLogger(__name__)
if os.getenv("DERIVA_PY_TEST_VERBOSE"):
    logger.setLevel(logging.DEBUG)
    logger.addHandler(logging.StreamHandler())


def setUpModule():
    global test_catalog
    logger.debug("setUpModule begin")
    credential = os.getenv("DERIVA_PY_TEST_CREDENTIAL") or get_credential(hostname)
    server = DerivaServer("https", hostname, credentials=credential)
    try:
        test_catalog = server.create_ermrest_catalog()
        model = test_catalog.getCatalogModel()
        create_ml_schema(model)
        create_domain_schema(model, SNAME_DOMAIN)
    except Exception:
        # on failure, delete catalog and re-raise exception
        test_catalog.delete_ermrest_catalog(really=True)
    logger.debug("setUpModule  done")


def tearDownModule():
    logger.debug("tearDownModule begin")
    try:
        test_catalog.delete_ermrest_catalog(really=True)
    except Exception:
        pass
    logger.debug("tearDownModule done")


class TestUpload(unittest.TestCase):
    def setUp(self):
        print(f"Calling setup {test_catalog.catalog_id}")
        self.ml_instance = DerivaML(
            hostname=hostname,
            catalog_id=test_catalog.catalog_id,
            domain_schema=SNAME_DOMAIN,
        )

        self.domain_schema = self.ml_instance.model.schemas[SNAME_DOMAIN]
        self.model = self.ml_instance.model

    def tearDown(self):
        pass

    def test_upload_directory(self):
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        self.ml_instance.create_asset("FooBar")
        with TemporaryDirectory() as tmpdir:
            asset_dir = self.ml_instance.asset_dir("FooBar", prefix=tmpdir)
            for s in range(2):
                asset_file = asset_dir.create_file(f"test_{s}.txt", metadata={})
                with open(asset_file, "w+") as f:
                    f.write(f"Hello there {random()}\n")
            self.ml_instance.upload_assets(asset_dir)
        assets = list(
            self.ml_instance.catalog.getPathBuilder()
            .schemas[SNAME_DOMAIN]
            .tables["FooBar"]
            .entities()
            .fetch()
        )
        self.assertEqual(len(assets), 2)

    def test_upload_directory_metadata(self):
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        subject_path = (
            self.ml_instance.catalog.getPathBuilder()
            .schemas[SNAME_DOMAIN]
            .tables["Subject"]
        )
        ss = list(subject_path.insert([{"Name": f"Thing{t + 1}"} for t in range(2)]))
        print(list(subject_path.entities().fetch()))
        print(ss)
        with TemporaryDirectory() as tmpdir:
            image_dir = self.ml_instance.asset_dir("Image", prefix=tmpdir)
            for s in ss:
                image_file = image_dir.create_file(
                    f"test_{s['RID']}.txt", {"Subject": s["RID"]}
                )
                with open(image_file, "w+") as f:
                    f.write(f"Hello there {random()}\n")
            self.ml_instance.upload_assets(image_dir)
        assets = list(
            self.ml_instance.catalog.getPathBuilder()
            .schemas[SNAME_DOMAIN]
            .tables["Image"]
            .entities()
            .fetch()
        )
        print(assets)
        self.assertIn(assets[0]["Subject"], [s["RID"] for s in ss])
        self.assertEqual(len(assets), 2)

    def test_upload_execution_metatable(self):
        pass

    def test_upload_execution_assets(self):
        metadata_file = execution_metadata_dir(MetataType.foo)
        with open(metadata_dir) as f:
            write()

        self, ml_instance.upload_execution_outputs(metadata_file)

    def test_upload_execution_outputs(self):
        pass



# Tests for the datapath module.
#
# Environment variables:
#  DERIVA_PY_TEST_HOSTNAME: hostname of the test server
#  DERIVA_PY_TEST_CREDENTIAL: user credential, if none, it will attempt to get credentail for given hostname
#  DERIVA_PY_TEST_VERBOSE: set for verbose logging output to stdout
import logging
import os
import sys
import unittest

from deriva.core import DerivaServer, ErmrestCatalog, get_credential
from typing import Optional
from deriva_ml.schema_setup.create_schema import initialize_ml_schema, create_ml_schema
from deriva_ml.demo_catalog import create_demo_catalog, populate_demo_catalog

try:
    from pandas import DataFrame

    HAS_PANDAS = True
except ImportError:
    HAS_PANDAS = False

SNAME_DOMAIN = "ml-test"

hostname = os.getenv("DERIVA_PY_TEST_HOSTNAME")
logger = logging.getLogger(__name__)
if os.getenv("DERIVA_PY_TEST_VERBOSE"):
    logger.setLevel(logging.DEBUG)
    logger.addHandler(logging.StreamHandler())

    logger.debug("setUpModule begin")
    credential = os.getenv("DERIVA_PY_TEST_CREDENTIAL") or get_credential(hostname)
    try:
        test_catalog = create_demo_catalog(hostname)
    except Exception:
        # on failure, delete catalog and re-raise exception
        test_catalog.delete_ermrest_catalog(really=True)
        raise
    logger.debug("setUpModule  done")

# test_catalog = create_demo_catalog(hostname)


class DerivaMLTest(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        print("callign DerivaMLTest")
        self.test_catalog = test_catalog
        super().__init__(*args, **kwargs)


# Example assertion

# Discover and run all test cases in the "tests" directory
test_loader = unittest.TestLoader()
test_suite = test_loader.discover("tests")

print(test_suite)
test_runner = unittest.TextTestRunner()
test_runner.run(test_suite)


if __name__ == "__main__":
    sys.exit(unittest.main())



import unittest

from deriva_ml import DerivaML, RID, DerivaMLException
from deriva.core import DerivaServer, get_credential
import os
from deriva_ml.demo_catalog import (
    create_ml_schema,
    create_domain_schema,
    reset_demo_catalog,
)
import logging

hostname = os.getenv("DERIVA_PY_TEST_HOSTNAME")
SNAME = os.getenv("DERIVA_PY_TEST_SNAME")
SNAME_DOMAIN = "deriva-test"

logger = logging.getLogger(__name__)
if os.getenv("DERIVA_PY_TEST_VERBOSE"):
    logger.setLevel(logging.DEBUG)
    logger.addHandler(logging.StreamHandler())


def setUpModule():
    global test_catalog
    logger.debug("setUpModule begin")
    credential = os.getenv("DERIVA_PY_TEST_CREDENTIAL") or get_credential(hostname)
    server = DerivaServer("https", hostname, credentials=credential)
    try:
        test_catalog = server.create_ermrest_catalog()
        model = test_catalog.getCatalogModel()
        create_ml_schema(model)
        create_domain_schema(model, SNAME_DOMAIN)
    except Exception:
        # on failure, delete catalog and re-raise exception
        test_catalog.delete_ermrest_catalog(really=True)
    logger.debug("setUpModule  done")


def tearDownModule():
    logger.debug("tearDownModule begin")
    try:
        test_catalog.delete_ermrest_catalog(really=True)
    except Exception:
        pass
    logger.debug("tearDownModule done")


class TestDataset(unittest.TestCase):
    def setUp(self):
        self.ml_instance = DerivaML(
            hostname, test_catalog.catalog_id, SNAME_DOMAIN, None, None, "1"
        )
        self.domain_schema = self.ml_instance.model.schemas[SNAME_DOMAIN]
        self.model = self.ml_instance.model

    def tearDown(self):
        pass

    def test_add_element_type(self):
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        self.ml_instance.add_dataset_element_type("Subject")
        self.assertEqual(len(list(self.ml_instance.list_dataset_element_types())), 2)

    def test_create_dataset(self) -> RID:
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        self.ml_instance.add_dataset_element_type("Subject")
        type_rid = self.ml_instance.add_term(
            "Dataset_Type", "TestSet", description="A test"
        )
        dataset_rid = self.ml_instance.create_dataset(
            type_rid.name, description="A Dataset"
        )
        self.assertEqual(len(list(self.ml_instance.find_datasets())), 1)
        return dataset_rid

    def test_add_dataset_members(self):
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        subject_path = (
            self.ml_instance.catalog.getPathBuilder()
            .schemas[SNAME_DOMAIN]
            .tables["Subject"]
        )
        dataset_rid = self.test_create_dataset()
        subject_path.insert([{"Name": f"Thing{t + 1}"} for t in range(5)])
        subject_rids = [i["RID"] for i in subject_path.entities().fetch()]
        self.ml_instance.add_dataset_members(
            dataset_rid=dataset_rid, members=subject_rids
        )
        self.assertEqual(
            len(self.ml_instance.list_dataset_members(dataset_rid)["Subject"]),
            len(subject_rids),
        )
        self.assertEqual(len(self.ml_instance.dataset_history(dataset_rid)), 2)
        self.assertEqual(str(self.ml_instance.dataset_version(dataset_rid)), "0.2.0")

    def test_delete_dataset_members(self):
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        subject_path = (
            self.ml_instance.catalog.getPathBuilder()
            .schemas[SNAME_DOMAIN]
            .tables["Subject"]
        )
        dataset_rid = self.test_create_dataset()
        subject_path.insert([{"Name": f"Thing{t + 1}"} for t in range(5)])
        subject_rids = [i["RID"] for i in subject_path.entities().fetch()]
        self.ml_instance.add_dataset_members(
            dataset_rid=dataset_rid, members=subject_rids
        )

        subject_rids = [
            s["RID"]
            for s in self.ml_instance.list_dataset_members(dataset_rid)["Subject"]
        ]
        self.assertEqual(len(subject_rids), 5)
        self.ml_instance.delete_dataset_members(dataset_rid, subject_rids[0:2])
        subject_rids = self.ml_instance.list_dataset_members(dataset_rid)["Subject"]
        self.assertEqual(len(subject_rids), 3)
        self.assertEqual(len(self.ml_instance.dataset_history(dataset_rid)), 3)
        self.assertEqual(str(self.ml_instance.dataset_version(dataset_rid)), "0.3.0")

    def test_delete_dataset(self):
        subject_path = (
            self.ml_instance.catalog.getPathBuilder()
            .schemas[SNAME_DOMAIN]
            .tables["Subject"]
        )
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        reset_demo_catalog(self.ml_instance, SNAME_DOMAIN)
        self.ml_instance.add_dataset_element_type("Subject")
        type_rid = self.ml_instance.add_term(
            "Dataset_Type", "TestSet", description="A test"
        )
        dataset_rids = [
            self.ml_instance.create_dataset(type_rid.name, description="A Dataset")
            for i in range(5)
        ]
        subject_path.insert([{"Name": f"Thing{t + 1}"} for t in range(5)])
        subject_rids = [i["RID"] for i in subject_path.entities().fetch()]
        self.ml_instance.add_dataset_members(
            dataset_rid=dataset_rids[0], members=subject_rids
        )
        self.assertEqual(5, len(self.ml_instance.find_datasets()))
        self.ml_instance.delete_dataset(dataset_rids[0])
        self.assertEqual(4, len(self.ml_instance.find_datasets()))
        self.assertEqual(5, len(self.ml_instance.find_datasets(deleted=True)))
        self.ml_instance.delete_dataset(dataset_rids[1])
        self.assertRaises(
            DerivaMLException, self.ml_instance.list_dataset_members, dataset_rids[0]
        )



import unittest
from deriva_ml import DerivaML, DerivaMLException, ColumnDefinition, BuiltinTypes
from deriva.core import DerivaServer, get_credential
import os
from deriva_ml.demo_catalog import (
    create_ml_schema,
    create_domain_schema,
)
import logging

hostname = os.getenv("DERIVA_PY_TEST_HOSTNAME")
SNAME = os.getenv("DERIVA_PY_TEST_SNAME")
SNAME_DOMAIN = "deriva-test"

logger = logging.getLogger(__name__)
if os.getenv("DERIVA_PY_TEST_VERBOSE"):
    logger.setLevel(logging.DEBUG)
    logger.addHandler(logging.StreamHandler())


def setUpModule():
    print("Calling setupModule")
    global test_catalog
    logger.debug("setUpModule begin")
    credential = os.getenv("DERIVA_PY_TEST_CREDENTIAL") or get_credential(hostname)
    server = DerivaServer("https", hostname, credentials=credential)
    try:
        test_catalog = server.create_ermrest_catalog()
        model = test_catalog.getCatalogModel()
        create_ml_schema(model)
        create_domain_schema(model, SNAME_DOMAIN)
    except Exception:
        # on failure, delete catalog and re-raise exception
        test_catalog.delete_ermrest_catalog(really=True)
    logger.debug("setUpModule  done")


def tearDownModule():
    logger.debug("tearDownModule begin")
    try:
        test_catalog.delete_ermrest_catalog(really=True)
    except Exception:
        pass
    logger.debug("tearDownModule done")


@unittest.skipUnless(hostname, "Test host not specified")
class TestVocabulary(unittest.TestCase):

    def setUp(self):
        self.ml_instance = DerivaML(
            hostname, test_catalog.catalog_id, SNAME_DOMAIN, None, None, "1"
        )
        self.domain_schema = self.ml_instance.model.schemas[SNAME_DOMAIN]
        self.model = self.ml_instance.model

    def tearDown(self):
        pass

    def test_find_vocabularies(self):
        # Look for a known vocabulary in the deriva-ml schema
        self.assertIn(
            "Dataset_Type", [v.name for v in self.ml_instance.find_vocabularies()]
        )

    def test_is_vocabulary(self):
        # Test the vocabulary table predicates.
        self.assertTrue(self.ml_instance.model.is_vocabulary("Dataset_Type"))
        self.assertFalse(self.ml_instance.model.is_vocabulary("Dataset"))
        self.assertRaises(
            DerivaMLException, self.ml_instance.model.is_vocabulary, "FooBar"
        )

    def test_create_vocabulary(self):
        self.ml_instance.create_vocabulary("CV1", "A vocab")
        self.assertIn("CV1", [v.name for v in self.ml_instance.find_vocabularies()])
        self.assertTrue(self.ml_instance.model.is_vocabulary("Dataset_Type"))

    def test_add_term(self):
        self.ml_instance.create_vocabulary("CV2", "A vocab")
        self.assertEqual(len(self.ml_instance.list_vocabulary_terms("CV2")), 0)
        term = self.ml_instance.add_term("CV2", "T1", description="A vocab")
        self.assertEqual(len(self.ml_instance.list_vocabulary_terms("CV2")), 1)
        self.assertEqual(term.name, self.ml_instance.lookup_term("CV2", "T1").name)

        # Check for redundant terms.
        with self.assertRaises(DerivaMLException) as context:
            self.ml_instance.add_term(
                "CV2", "T1", description="A vocab", exists_ok=False
            )
        self.assertEqual(
            "T1", self.ml_instance.add_term("CV2", "T1", description="A vocab").name
        )

    def test_find_assets(self):
        self.assertTrue(self.ml_instance.model.is_asset("Execution_Asset"))
        self.assertFalse(self.ml_instance.model.is_asset("Dataset"))
        self.assertIn(
            "Execution_Asset", [a.name for a in self.ml_instance.find_assets()]
        )

    def test_is_assoc(self):
        print("Calling test_is_assoc begin")
        self.assertTrue(self.ml_instance.model.is_association("Dataset_Dataset"))
        self.assertFalse(self.ml_instance.model.is_association("Dataset"))

    def test_create_assets(self):
        self.ml_instance.create_asset("FooAsset")
        self.assertIn("FooAsset", [a.name for a in self.ml_instance.find_assets()])
        self.ml_instance.create_asset(
            "BarAsset",
            column_defs=[ColumnDefinition(name="foo", type=BuiltinTypes.int4)],
        )
        self.assertIn("BarAsset", [a.name for a in self.ml_instance.find_assets()])
        self.assertEqual(1, len(self.ml_instance.model.asset_metadata("BarAsset")))
